# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['interpolation_search', 'ainterpolation_search', 'batched']

# %% ../nbs/00_core.ipynb 3
import asyncio
from itertools import islice

# %% ../nbs/00_core.ipynb 5
def interpolation_search(o, target, low=None, high=None, how='left'):
    """Interpolation search algorithm to find the closest value to `target` in the sorted list `o`."""

    assert how in ['left', 'right']

    if low is None:
        low = 0
    if high is None:
        high = len(o)
    
    assert o[low] <= target, f'Target {target} is smaller than the lowest value in the list, {o[low]}. low={low}'
    assert target <= o[high], f'Target {target} is larger than the highest value in the list, {o[high]}. high={high}'

    pos = low + ((int(target) - o[low]) * (high - low)) // (o[high] - o[low])

    if o[pos] < target:
        if o[pos + 1] > target:
            if how=='left':
                return pos
            else:
                return pos + 1
        return interpolation_search(o, target, pos + 1, high, how=how)
    else:
        if o[pos - 1] < target:
            if how=='right':
                return pos
            else:
                return pos - 1
        return interpolation_search(o, target, low, pos - 1, how=how)

# %% ../nbs/00_core.ipynb 6
async def ainterpolation_search(o, target, low=None, high=None, how='left'):
    """Interpolation search algorithm to find the closest value to `target` in the sorted list `o`."""

    assert how in ['left', 'right']

    if low is None:
        low = 0
    if high is None:
        high = await o.alen()
    
    o_low, o_high = await asyncio.gather(o.agetitem(low), o.agetitem(high))
    assert o_low <= target, f'Target {target} is smaller than the lowest value in the list, {o_low}. low={low}'
    assert target <= o_high, f'Target {target} is larger than the highest value in the list, {o_high}. high={high}'

    pos = low + ((int(target) - o_low) * (high - low)) // (o_high - o_low)
    o_pos_m1, o_pos, o_pos_p1 = await asyncio.gather(o.agetitem(pos - 1), o.agetitem(pos), o.agetitem(pos + 1))
    if o_pos < target:
        if o_pos_p1 > target:
            if how=='left':
                return pos
            else:
                return pos + 1
        return await ainterpolation_search(o, target, pos + 1, high, how=how)
    else:
        if o_pos_m1 < target:
            if how=='right':
                return pos
            else:
                return pos - 1
        return await ainterpolation_search(o, target, low, pos - 1, how=how)

# %% ../nbs/00_core.ipynb 7
def batched(iterable, n):
    "Batch data into tuples of length n. The last batch may be shorter."
    # batched('ABCDEFG', 3) --> ABC DEF G

    if n < 1:
        raise ValueError('n must be at least one')
    it = iter(iterable)
    while batch := tuple(islice(it, n)):
        yield batch
